--
-- CutterDensityHeightPickupFix
-- Specialization to allow cutter pickup area function to collect also fillTypes with the same name as the fruitType if the fruitType has no windrow
--
-- @author  Stefan Maurus
-- @date  13/05/2020
--
-- @author Peppe978 - FS22 ready
-- @date  23/07/2022
--
-- Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.

-- @category Specializations
CutterDensityHeightPickupFix = {}

---Checks if all prerequisite specializations are loaded
-- @param table specializations specializations
-- @return boolean hasPrerequisite true if all prerequisite specializations are loaded
-- @includeCode
function CutterDensityHeightPickupFix.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Cutter, specializations)
end

---
-- @includeCode
function CutterDensityHeightPickupFix.registerOverwrittenFunctions(vehicleType)
    SpecializationUtil.registerOverwrittenFunction(vehicleType, "processPickupCutterArea", CutterDensityHeightPickupFix.processPickupCutterArea)
    SpecializationUtil.registerOverwrittenFunction(vehicleType, "getAllowCutterAIFruitRequirements", CutterDensityHeightPickupFix.getAllowCutterAIFruitRequirements)
end

---
-- @includeCode
function CutterDensityHeightPickupFix.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", CutterDensityHeightPickupFix)
end

---
-- @includeCode
function CutterDensityHeightPickupFix:onLoad(savegame)
    local spec = self.spec_cutter
    if spec.fruitTypes ~= nil then
        for _, fruitTypeIndex in ipairs(spec.fruitTypes) do
            local fillType = g_fruitTypeManager:getFillTypeIndexByFruitTypeIndex(fruitTypeIndex)

            if fillType ~= nil then
                self:setAIDensityHeightTypeRequirements(fillType)
            end
        end
    end
end

---
-- @includeCode
function CutterDensityHeightPickupFix:processPickupCutterArea(superFunc, workArea, dt)
    local spec = self.spec_cutter

    if spec.workAreaParameters.combineVehicle ~= nil then
        local sx, sy, sz = getWorldTranslation(workArea.start)
        local wx, wy, wz = getWorldTranslation(workArea.width)
        local hx, hy, hz = getWorldTranslation(workArea.height)

        local lsx, lsy, lsz, lex, ley, lez, lineRadius = DensityMapHeightUtil.getLineByAreaDimensions(sx, sy, sz, wx, wy, wz, hx, hy, hz)

        for _, fruitType in ipairs(spec.workAreaParameters.fruitTypesToUse) do
            local fruitDesc = g_fruitTypeManager:getFruitTypeByIndex(fruitType)
            local fillTypeIndex = g_fruitTypeManager:getWindrowFillTypeIndexByFruitTypeIndex(fruitType)

            if fillTypeIndex == nil then
                fillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(fruitDesc.name)
            end

            if fillTypeIndex ~= nil then
                local pickedUpLiters = -DensityMapHeightUtil.tipToGroundAroundLine(self, -math.huge, fillTypeIndex, lsx, lsy, lsz, lex, ley, lez, lineRadius, lineRadius, nil, false, nil)
                
                if self.isServer then
                    if pickedUpLiters > 0 then
                        local literPerSqm = fruitDesc.literPerSqm
                        local lastCutterArea = pickedUpLiters / (g_currentMission:getFruitPixelsToSqm() * literPerSqm);

                        if fruitType ~= spec.currentInputFruitType then
                            spec.currentInputFruitType = fruitType
                            spec.currentOutputFillType = g_fruitTypeManager:getFillTypeIndexByFruitTypeIndex(spec.currentInputFruitType)
                        end

                        spec.useWindrow = true
                        spec.currentInputFillType = fillTypeIndex
                        spec.workAreaParameters.lastFruitType = fruitType
                        spec.workAreaParameters.lastRealArea  = spec.workAreaParameters.lastRealArea + lastCutterArea
                        spec.workAreaParameters.lastThreshedArea  = spec.workAreaParameters.lastThreshedArea + lastCutterArea
                        spec.workAreaParameters.lastStatsArea  = spec.workAreaParameters.lastStatsArea + lastCutterArea
                        spec.workAreaParameters.lastArea      = spec.workAreaParameters.lastArea + lastCutterArea
                        spec.stoneLastState = FSDensityMapUtil.getStoneArea(sx, sz, wx, wz, hx, hz)
                        spec.isWorking = true
                        break
                    end
                end
            end
        end
    end

    return spec.workAreaParameters.lastRealArea, spec.workAreaParameters.lastArea
end

---
-- @includeCode
function CutterDensityHeightPickupFix:getAllowCutterAIFruitRequirements(superFunc)
    return false -- block changing of required fruit type since we are doing this on our own
end
